/**
 * -----------------------------------------------------
 * File        sqlback.sp
 * Authors     David <popoklopsi> Ordnung
 * License     GPLv3
 * Web         http://popoklopsi.de
 * -----------------------------------------------------
 * 
 * Copyright (C) 2012-2014 David <popoklopsi> Ordnung
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>
 */

// Semicolon
#pragma semicolon 1





// Get current databe version
void sqlback_getDatabaseVersion()
{
	if (sqllib_db != INVALID_HANDLE)
	{
		char query[128];

		
		// Get highest version
		Format(query, sizeof(query), g_sSelectVersionQuery, g_sTableName);
		
		StammLog(true, "Execute %s", query);

		SQL_TQuery(sqllib_db, sqlback_getVersion, query);
	}
}





// check version
bool sqlback_isVersionNewer(char[] version)
{
	//char version2[strlen(version) + 1];
	//char[] version2;
	char[] version2 = new char[strlen(version) + 1];

	// Replace dot
	strcopy(version2, strlen(version) + 1, version);
	ReplaceString(version2, strlen(version) + 1, ".", "");

	// Check
	return (strcmp(g_sDatabaseVersion, version2, false) < 0);
}





// get the version
public void sqlback_getVersion(Handle owner, Handle hndl, const char[] error, any data)
{
	// Found a value?
	if (hndl != INVALID_HANDLE && StrEqual(error, "") && SQL_FetchRow(hndl))
	{
		SQL_FetchString(hndl, 0, g_sDatabaseVersion, sizeof(g_sDatabaseVersion));
	}
	else
	{
		Format(g_sDatabaseVersion, sizeof(g_sDatabaseVersion), "000");
	}


	// Check if we need to modify
	sqlback_ModifyTableBackwards();
}




// Sync steamid game indepentend
void sqlback_syncSteamid(int client, const char[] version)
{
	// Only for versions < 2.1
	if (sqllib_db != INVALID_HANDLE && StringToInt(version[0]) <= 2 && StringToInt(version[2]) < 1 && StringToInt(version[3]) <= 9)
	{
		char query[128];
		char steamid[64];
		

		// Get int steamid and replace
		clientlib_getSteamid(client, steamid, sizeof(steamid));
		ReplaceString(steamid, sizeof(steamid), "STEAM_0:", "STEAM_1:");
		


		// get points of maybe existing STEAM_1: entry
		Format(query, sizeof(query), g_sSelectPointsQuery, g_sTableName, steamid);
		

		StammLog(true, "Execute %s", query);


		SQL_TQuery(sqllib_db, sqlback_syncSteamid1, query, GetClientUserId(client));
	}
}





// SQL handler
public void sqlback_syncSteamid1(Handle owner, Handle hndl, const char[] error, any userid)
{
	int client = GetClientOfUserId(userid);
	

	// Found a entry?
	if (hndl != INVALID_HANDLE && StrEqual(error, "") && SQL_FetchRow(hndl) && clientlib_isValidClient_PRE(client))
	{
		char query[128];
		char steamid[64];
		

		// Updated client points of STEAM_1: entry
		clientlib_getSteamid(client, steamid, sizeof(steamid));
		ReplaceString(steamid, sizeof(steamid), "STEAM_0:", "STEAM_1:");
		

		pointlib_GivePlayerPoints(client, SQL_FetchInt(hndl, 0), false);


		// Delete STEAM_1: entry
		Format(query, sizeof(query), g_sDeletePlayerQuery, g_sTableName, steamid);
		
		StammLog(true, "Execute %s", query);

		SQL_TQuery(sqllib_db, sqllib_SQLErrorCheckCallback, query);
	}
}






// Check for needed modify
void sqlback_ModifyTableBackwards()
{
	char query[256];


	// Version <= 2.2
	if (sqlback_isVersionNewer("2.20"))
	{
		// Remove Happy table
		Format(query, sizeof(query), g_sDropHappyTable, g_sTableName);
		
		StammLog(true, "Execute %s", query);

		SQL_TQuery(sqllib_db, sqllib_SQLErrorCheckCallback2, query);
	}

	// Version <= 2.15
	if (sqlback_isVersionNewer("2.16"))
	{
		// Add admin
		Format(query, sizeof(query), g_sAlterAdminQuery, g_sTableName);
		
		StammLog(true, "Execute %s", query);

		SQL_TQuery(sqllib_db, sqllib_SQLErrorCheckCallback2, query);
	}


	// Version <= 2.10
	if (sqlback_isVersionNewer("2.11"))
	{
		// Add last visit
		Format(query, sizeof(query), g_sAlterLastVisitQuery, g_sTableName, GetTime());
		
		StammLog(true, "Execute %s", query);

		SQL_TQuery(sqllib_db, sqllib_SQLErrorCheckCallback2, query);
	}



	// Version < 2.1
	if (sqlback_isVersionNewer("2.10"))
	{
		if (sqllib_db != INVALID_HANDLE)
		{
			// Add version column
			Format(query, sizeof(query), g_sAlterVersionQuery, g_sTableName);
			
			StammLog(true, "Execute %s", query);

			SQL_TQuery(sqllib_db, sqllib_SQLErrorCheckCallback2, query);



			// Drop payed	
			Format(query, sizeof(query), g_sAlterPayedQuery, g_sTableName);
			
			StammLog(true, "Execute %s", query);

			SQL_TQuery(sqllib_db, sqllib_SQLErrorCheckCallback2, query);
			


			// Maybe we came from and old 1. version?
			Format(query, sizeof(query), g_sSelectAllPointsQuery, g_sTableName);
			
			StammLog(true, "Execute %s", query);

			SQL_TQuery(sqllib_db, sqlback_SQLModify1, query);
		}
	}
	else
	{
		// Start stamm
		stammStarted();
	}
}







// Check for very old version
public void sqlback_SQLModify1(Handle owner, Handle hndl, const char[] error, any data)
{
	if (hndl != INVALID_HANDLE && StrEqual(error, "") && SQL_FetchRow(hndl))
	{
		char query[600];
		char ident[32];


		// Get Driver
		Handle driver = SQL_ReadDriver(sqllib_db, ident, sizeof(ident));

		// Create int table as backup
		if (driver != INVALID_HANDLE && StrEqual(ident, "mysql"))
		{
			Format(query, sizeof(query), g_sCreateBackupQueryMySQL, g_sTableName, GetTime());
		}
		else
		{
			Format(query, sizeof(query), g_sCreateBackupQuery, g_sTableName, GetTime());
		}
		

		StammLog(true, "Execute %s", query);

		SQL_TQuery(sqllib_db, sqlback_SQLModify2, query);
	}
	else
	{
		// No old version -> Stamm start
		stammStarted();
	}
}




// Convert from old database
public void sqlback_SQLModify2(Handle owner, Handle hndl, const char[] error, any data)
{
	if (hndl != INVALID_HANDLE)
	{
		char query[600];
		
		// Insert from old database
		Format(query, sizeof(query), g_sInsertBackupQuery, g_sTableName, g_sTableName);
	

		StammLog(true, "Execute %s", query);

		SQL_TQuery(sqllib_db, sqlback_SQLModify3, query);
	}
	else
	{
		// Terrible error
		SetFailState("Error converting Stamm to the newest database structur. Error:   %s", error);
	}
}






// Next step
public void sqlback_SQLModify3(Handle owner, Handle hndl, const char[] error, any data)
{
	if (hndl != INVALID_HANDLE)
	{
		char query[128];


		// Rename old database to old
		Format(query, sizeof(query), g_sAlterRenameQuery, g_sTableName, g_sTableName);
		
		StammLog(true, "Execute %s", query);

		SQL_TQuery(sqllib_db, sqlback_SQLModify4, query);
	}
	else
	{
		SetFailState("Error converting Stamm to the newest database structur. Error:   %s", error);
	}
}







// Next step
public void sqlback_SQLModify4(Handle owner, Handle hndl, const char[] error, any data)
{
	if (hndl != INVALID_HANDLE)
	{
		char query[128];
		


		// Make int database to main
		Format(query, sizeof(query), g_sAlterRename2Query, g_sTableName, g_sTableName);
		
		StammLog(true, "Execute %s", query);

		SQL_TQuery(sqllib_db, sqlback_SQLModify5, query);
	}
	else
	{
		SetFailState("Error converting Stamm to the newest database structur. Error:   %s", error);
	}
}






public void sqlback_SQLModify5(Handle owner, Handle hndl, const char[] error, any data)
{
	if (hndl != INVALID_HANDLE)
	{
		// Start stamm now
		stammStarted();
	}
	else
	{
		SetFailState("Error converting Stamm to the newest database structur. Error:   %s", error);
	}
}