/**
 * -----------------------------------------------------
 * File        stamm_chat_messages.sp
 * Authors     David <popoklopsi> Ordnung
 * License     GPLv3
 * Web         http://popoklopsi.de
 * -----------------------------------------------------
 * 
 * Copyright (C) 2012-2014 David <popoklopsi> Ordnung
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>
 */


// Icnludes
#include <sourcemod>

#undef REQUIRE_PLUGIN
#include <stamm>

#pragma semicolon 1
#pragma newdecls required




int g_iWelcome = -1;
int g_iLeave = -1;




public Plugin myinfo =
{
	name = "Stamm Feature Chat Messages",
	author = "Popoklopsi",
	version = "${-version-}", // Version is replaced by GitLab runner due the build phase
	description = "Give VIP's VIP Chat and Message",
	url = "https://gitlab.com/PushTheLimits/Sourcemod/Stamm/wikis"
};




// ADd Feature
public void OnAllPluginsLoaded()
{
	if (!STAMM_IsAvailable()) 
	{
		SetFailState("Can't Load Feature, Stamm is not installed!");
	}


	STAMM_LoadTranslation();	
	STAMM_RegisterFeature("VIP Chat Messages");
}




// Feature loaded
public void STAMM_OnFeatureLoaded(const char[] basename)
{
	// Get Blocks
	g_iWelcome = STAMM_GetBlockOfName("welcome");
	g_iLeave = STAMM_GetBlockOfName("leave");
	

	if (g_iWelcome == -1 && g_iLeave == -1)
	{
		SetFailState("Found neither block welcome nor block leave!");
	}
}




// Add descriptions
public void STAMM_OnClientRequestFeatureInfo(int client, int block, Handle &array)
{
	char fmt[256];
	
	if (block == g_iWelcome)
	{
		Format(fmt, sizeof(fmt), "%T", "GetWelcomeMessages", client);
		
		PushArrayString(array, fmt);
	}

	if (block == g_iLeave)
	{
		Format(fmt, sizeof(fmt), "%T", "GetLeaveMessages", client);
		
		PushArrayString(array, fmt);
	}
}




// Client Ready
public void STAMM_OnClientReady(int client)
{
	char name[MAX_NAME_LENGTH + 1];
	char tag[64];


	GetClientName(client, name, sizeof(name));
	STAMM_GetTag(tag, sizeof(tag));


	// Gets a welcome message?
	if (g_iWelcome != -1 && STAMM_IsClientValid(client) && STAMM_HaveClientFeature(client, g_iWelcome))
	{
		STAMM_PrintToChatAll("%s %t", tag, "WelcomeMessage", name);
	}
}


// Client Disonnect
public void OnClientDisconnect(int client)
{
	if (STAMM_IsClientValid(client) && g_iLeave != -1)
	{
		char name[MAX_NAME_LENGTH + 1];
		char tag[64];


		GetClientName(client, name, sizeof(name));
		STAMM_GetTag(tag, sizeof(tag));


		// Gets a leave message?
		if (STAMM_HaveClientFeature(client, g_iLeave))
		{
			STAMM_PrintToChatAll("%s %t", tag, "LeaveMessage", name);
		}
	}
}