/**
 * -----------------------------------------------------
 * File        stamm_flagpoints.sp
 * Authors     David <popoklopsi> Ordnung
 * License     GPLv3
 * Web         http://popoklopsi.de
 * -----------------------------------------------------
 * 
 * Copyright (C) 2012-2014 David <popoklopsi> Ordnung
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>
 */


// Includes
#include <sourcemod>
#include <autoexecconfig>

#undef REQUIRE_PLUGIN
#include <stamm>

#pragma semicolon 1
#pragma newdecls required



Handle  g_hFlagNeed;



public Plugin myinfo =
{
	name = "Stamm Feature FlagPoints",
	author = "Popoklopsi",
	version = "${-version-}", // Version is replaced by GitLab runner due the build phase
	description = "Give only points to players with a specific flag",
	url = "https://gitlab.com/PushTheLimits/Sourcemod/Stamm/wikis"
};



public void STAMM_OnFeatureLoaded(const char[] basename)
{
}



// Create config
public void OnPluginStart()
{
	AutoExecConfig_SetFile("flagpoints", "stamm/features");
	AutoExecConfig_SetCreateFile(true);

	g_hFlagNeed = AutoExecConfig_CreateConVar("flag_need", "s", "Flag string a player needs to collect points");
	
	AutoExecConfig_CleanFile();
	AutoExecConfig_ExecuteFile();
}



// Add Feature
public void OnAllPluginsLoaded()
{
	if (!STAMM_IsAvailable()) 
	{
		SetFailState("Can't Load Feature, Stamm is not installed!");
	}


	// Load Translation
	STAMM_LoadTranslation();
	STAMM_RegisterFeature("VIP FlagPoints");
}





// Stop non VIP's getting points
public Action STAMM_OnClientGetPoints_PRE(int client, int &number)
{
	char tag[64];
	char flagNeed[32];


	GetConVarString(g_hFlagNeed, flagNeed, sizeof(flagNeed));


	if (((GetUserFlagBits(client) & ReadFlagString(flagNeed) == ReadFlagString(flagNeed)) || GetUserFlagBits(client) & ADMFLAG_ROOT))
	{
		return Plugin_Continue;
	}
	else
	{
		STAMM_GetTag(tag, sizeof(tag));

		STAMM_PrintToChat(client, "%s %t", tag, "NoPoints", flagNeed);
	}

	return Plugin_Handled;
}