/**
 * -----------------------------------------------------
 * File        stamm_lessgravity.sp
 * Authors     David <popoklopsi> Ordnung
 * License     GPLv3
 * Web         http://popoklopsi.de
 * -----------------------------------------------------
 * 
 * Copyright (C) 2012-2014 David <popoklopsi> Ordnung
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>
 */


// Includes
#include <sourcemod>
#include <autoexecconfig>

#undef REQUIRE_PLUGIN
#include <stamm>

#pragma semicolon 1
#pragma newdecls required



Handle  g_hGrav;




// Details
public Plugin myinfo =
{
	name = "Stamm Feature LessGravity",
	author = "Popoklopsi",
	version = "${-version-}", // Version is replaced by GitLab runner due the build phase
	description = "Give VIP's less gravity",
	url = "https://gitlab.com/PushTheLimits/Sourcemod/Stamm/wikis"
};




// Add the Feature
public void OnAllPluginsLoaded()
{
	if (!STAMM_IsAvailable()) 
	{
		SetFailState("Can't Load Feature, Stamm is not installed!");
	}

	STAMM_LoadTranslation();
	STAMM_RegisterFeature("VIP Less Gravity");
}




// Create the config
public void OnPluginStart()
{
	HookEvent("player_spawn", PlayerSpawn);


	AutoExecConfig_SetFile("lessgravity", "stamm/features");
	AutoExecConfig_SetCreateFile(true);

	g_hGrav = AutoExecConfig_CreateConVar("gravity_decrease", "5", "Gravity decrease in percent each block!");
	
	AutoExecConfig_CleanFile();
	AutoExecConfig_ExecuteFile();
}




public void STAMM_OnFeatureLoaded(const char[] basename)
{
}




// Add descriptions
public void STAMM_OnClientRequestFeatureInfo(int client, int block, Handle &array)
{
	char fmt[256];
	
	Format(fmt, sizeof(fmt), "%T", "GetLessGravity", client, GetConVarInt(g_hGrav) * block);
	
	PushArrayString(array, fmt);
}




// A Player spawned, change his gravity
public void PlayerSpawn(Handle event, char[] name, bool dontBroadcast)
{
	int client = GetClientOfUserId(GetEventInt(event, "userid"));
	
	// Players spawn without Gravity (not always)
	CreateTimer(0.1, Timer_ChangeGravity, client);
}


public Action Timer_ChangeGravity(Handle timer, any client)
{
	STAMM_OnClientChangedFeature(client, true, false);
}


public void STAMM_OnClientBecomeVip(int client, int oldlevel, int newlevel)
{
	STAMM_OnClientChangedFeature(client, true, false);
}



// Also change it, if he changed the state
public void STAMM_OnClientChangedFeature(int client, bool mode, bool isShop)
{
	if (STAMM_IsClientValid(client) && IsPlayerAlive(client))
	{
		float newGrav;
		int clientBlock;

		// Client want it
		if (mode)
		{
			// Get highest client block
			clientBlock = STAMM_GetClientBlock(client);

			// Have the client the block?
			if (clientBlock > 0)
			{
				// Calculate int gravity
				newGrav = 1.0 - float(GetConVarInt(g_hGrav)) / 100.0 * clientBlock;

				if (newGrav < 0.1) 
				{
					newGrav = 0.1;
				}

				SetEntityGravity(client, newGrav);
			}
		}
		else
		{
			// Else reset gravity
			SetEntityGravity(client, 1.0);
		}
	}
}



// When client is on a ladder gravity will go back to normal
public Action OnPlayerRunCmd(int client, int &buttons, int &impulse, float vel[3], float angles[3], int &weapon, int &subtype, int &cmdnum, int &tickcount, int &seed, int mouse[2])
{
	if (STAMM_IsClientValid(client))
	{
		static MoveType lastMove[MAXPLAYERS + 1] = MOVETYPE_NONE;
		MoveType current = GetEntityMoveType(client);

		if (current != MOVETYPE_LADDER && lastMove[client] == MOVETYPE_LADDER)
		{
			STAMM_OnClientChangedFeature(client, true, false);
		}

		lastMove[client] = current;
	}
}
