/**
 * -----------------------------------------------------
 * File        stamm_morespeed.sp
 * Authors     David <popoklopsi> Ordnung
 * License     GPLv3
 * Web         http://popoklopsi.de
 * -----------------------------------------------------
 * 
 * Copyright (C) 2012-2014 David <popoklopsi> Ordnung
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>
 */


// Includes
#include <sourcemod>
#include <sdktools>
#include <autoexecconfig>

#undef REQUIRE_PLUGIN
#include <stamm>

#pragma semicolon 1
#pragma newdecls required



Handle  g_hSpeed;




public Plugin myinfo =
{
	name = "Stamm Feature MoreSpeed",
	author = "Popoklopsi",
	version = "${-version-}", // Version is replaced by GitLab runner due the build phase
	description = "Give VIP's more speed",
	url = "https://gitlab.com/PushTheLimits/Sourcemod/Stamm/wikis"
};




// Add Feature
public void OnAllPluginsLoaded()
{
	if (!STAMM_IsAvailable()) 
	{
		SetFailState("Can't Load Feature, Stamm is not installed!");
	}

	STAMM_LoadTranslation();
	STAMM_RegisterFeature("VIP MoreSpeed");
}




// Create config
public void OnPluginStart()
{
	HookEvent("player_spawn", PlayerSpawn);


	AutoExecConfig_SetFile("morespeed", "stamm/features");
	AutoExecConfig_SetCreateFile(true);

	g_hSpeed = AutoExecConfig_CreateConVar("speed_increase", "20", "Speed increase in percent each block!");
	
	AutoExecConfig_CleanFile();
	AutoExecConfig_ExecuteFile();
}



public void STAMM_OnFeatureLoaded(const char[] basename)
{
}




// Add descriptions
public void STAMM_OnClientRequestFeatureInfo(int client, int block, Handle &array)
{
	char fmt[256];
	
	Format(fmt, sizeof(fmt), "%T", "GetMoreSpeed", client, GetConVarInt(g_hSpeed) * block);
	
	PushArrayString(array, fmt);
}





// Player spawned
public void PlayerSpawn(Handle event, char[] name, bool dontBroadcast)
{
	int client = GetClientOfUserId(GetEventInt(event, "userid"));
	
	STAMM_OnClientChangedFeature(client, true, false);
}



public void STAMM_OnClientBecomeVip(int client, int oldlevel, int newlevel)
{
	STAMM_OnClientChangedFeature(client, true, false);
}



// Client changed feature state
public void STAMM_OnClientChangedFeature(int client, bool mode, bool isShop)
{
	if (STAMM_IsClientValid(client) && IsPlayerAlive(client))
	{
		// He want more speed
		if (mode)
		{
			// Get highest client block
			int clientBlock = STAMM_GetClientBlock(client);


			// Client have block?
			if (clientBlock > 0)
			{
				// Set int speed of player
				float newSpeed;
				
				newSpeed = 1.0 + float(GetConVarInt(g_hSpeed)) / 100.0 * clientBlock;
				
				SetEntPropFloat(client, Prop_Data, "m_flLaggedMovementValue", newSpeed);
			}
		}
		else
		{
			// Set default speed
			SetEntPropFloat(client, Prop_Data, "m_flLaggedMovementValue", 1.0);
		}
	}
}